use strict;
use warnings;
use HTTP::Server::Simple::CGI;
use Net::MQTT::Simple;
use threads;
use threads::shared;

# MQTT settings
my $mqtt_server = "192.168.1.2";

# Shared variables for state handling
my $door_state :shared = "closed";
my $car_state :shared = "parked";
my $lock :shared;

# Subscribe to MQTT topics in a separate thread
threads->create(sub {
    my $mqtt = Net::MQTT::Simple->new($mqtt_server);
    
    $mqtt->run(
        "garage/door" => sub {
            my ($topic, $message) = @_;
            {
                lock($lock);
                $door_state = $message;
                #print "MQTT: ".$door_state."\n";
            }
        },
        "garage/car" => sub {
            my ($topic, $message) = @_;
            {
                lock($lock);
                $car_state = $message;
                #print "MQTT: ".$car_state."\n";
            }
        }
    );
})->detach();

# Web server definition
{
    package WebServer;
    use base 'HTTP::Server::Simple::CGI';
    use constant IMAGE_PATH => "/usr/local/share/garage";


    sub handle_request {
        my ($self, $cgi) = @_;
        if ($cgi->path_info() eq '/state') {
            print "HTTP/1.0 200 OK\r\n";
            print "Content-type: application/json\r\n\r\n";
            {
                lock($lock);
                print "{\"door\":\"$door_state\", \"car\":\"$car_state\"}\n";
            }
	} elsif ($cgi->path_info() =~ /\.png$/) { # Assuming all images are PNGs
		my $image_path = $cgi->path_info();
		#$image_path =~ s/^\///; # Remove leading slash for file reading

		if (-e IMAGE_PATH.$image_path) {
			print "HTTP/1.0 200 OK\r\n";
			print "Content-Type: image/png\r\n\r\n";
			open(my $image_fh, '<', IMAGE_PATH.$image_path) or die "Can't open $image_path: $!";
			binmode $image_fh;
			binmode STDOUT;
			print <$image_fh>;
			close($image_fh);
		} else {
			print "HTTP/1.0 404 Not Found\r\n";
			print "Content-Type: text/plain\r\n\r\n";
			print "File not found.";
		}
        } else {
            print "HTTP/1.0 200 OK\r\n";
            print "Content-type: text/html\r\n\r\n";
            print &main_page;
        }
    }

    sub main_page {
        return <<HTML;
<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>Garage State</title>
    <script>
      let currentDoorState = "closed";
      let currentCarState = "out"; // This is initialized to 'out' assuming the car is away initially. Adjust accordingly.

      function animateDoorChange(targetDoorState) {
        if (currentDoorState === targetDoorState) return;

        const direction = currentDoorState === "open" ? 1 : -1;
        let currentFrame = currentDoorState === "open" ? 2 : 5;
        const endFrame = targetDoorState === "open" ? 2 : 5;

        function step() {
          const imageName = "door_" + currentFrame + "_car_" + currentCarState + ".png";
          document.getElementById("garageImage").src = imageName;

          currentFrame += direction;

          if ((direction === 1 && currentFrame <= endFrame) || (direction === -1 && currentFrame >= endFrame)) {
            setTimeout(step, 500);
          } else {
            currentDoorState = targetDoorState;
            updateImage();
          }
        }

        step();
      }


      function updateImage() {
        fetch("/state")
          .then(response => response.json())
          .then(data => {
            const carImageNamePart = data.car === "parked" ? "in" : "out";

            // If only the door state changes, animate it
            if (currentDoorState !== data.door) {
                animateDoorChange(data.door);
            } else {
                const imageName = "door_" + data.door + "_car_" + carImageNamePart + ".png";
                document.getElementById("garageImage").src = imageName;
            }

            currentCarState = carImageNamePart;
          })
          .catch(error => {
            console.error("Error:", error);
          });
      }

      setInterval(updateImage, 5000);
    </script>
</head>
<body>
    <img id="garageImage" src="door_closed_car_in.png" alt="Garage State">
</body>
</html>
HTML
    }
}

WebServer->new(8081)->run();
